#include <Arduino.h>
#include <SPI.h>
#include "MPU6000.h"


#define  MPUREG_WHOAMI        0x75     // MPU 6000 registers
#define  MPUREG_SMPLRT_DIV    0x19     //
#define  MPUREG_CONFIG        0x1A
#define  MPUREG_GYRO_CONFIG   0x1B
#define  MPUREG_ACCEL_CONFIG  0x1C
#define  MPUREG_FIFO_EN       0x23
#define  MPUREG_INT_PIN_CFG   0x37
#define  MPUREG_INT_ENABLE    0x38
#define  MPUREG_INT_STATUS    0x3A
#define  MPUREG_ACCEL_XOUT_H  0x3B 
#define  MPUREG_ACCEL_XOUT_L  0x3C 
#define  MPUREG_ACCEL_YOUT_H  0x3D 
#define  MPUREG_ACCEL_YOUT_L  0x3E 
#define  MPUREG_ACCEL_ZOUT_H  0x3F 
#define  MPUREG_ACCEL_ZOUT_L  0x40 
#define  MPUREG_TEMP_OUT_H    0x41
#define  MPUREG_TEMP_OUT_L    0x42
#define  MPUREG_GYRO_XOUT_H   0x43 
#define  MPUREG_GYRO_XOUT_L   0x44 
#define  MPUREG_GYRO_YOUT_H   0x45 
#define  MPUREG_GYRO_YOUT_L   0x46 
#define  MPUREG_GYRO_ZOUT_H   0x47 
#define  MPUREG_GYRO_ZOUT_L   0x48 
#define  MPUREG_USER_CTRL     0x6A 
#define  MPUREG_PWR_MGMT_1    0x6B 
#define  MPUREG_PWR_MGMT_2    0x6C 
#define  MPUREG_FIFO_COUNTH   0x72
#define  MPUREG_FIFO_COUNTL   0x73
#define  MPUREG_FIFO_R_W      0x74

#define BIT_SLEEP                   0x40   // Configuration bits
#define BIT_H_RESET                 0x80   //
#define BITS_CLKSEL                 0x07
#define MPU_CLK_SEL_PLLGYROX        0x01
#define MPU_CLK_SEL_PLLGYROZ        0x03
#define MPU_EXT_SYNC_GYROX          0x02
#define BITS_FS_250DPS              0x00
#define BITS_FS_500DPS              0x08
#define BITS_FS_1000DPS             0x10
#define BITS_FS_2000DPS             0x18
#define BITS_FS_MASK                0x18
#define BITS_DLPF_CFG_256HZ_NOLPF2  0x00
#define BITS_DLPF_CFG_188HZ         0x01
#define BITS_DLPF_CFG_98HZ          0x02
#define BITS_DLPF_CFG_42HZ          0x03
#define BITS_DLPF_CFG_20HZ          0x04
#define BITS_DLPF_CFG_10HZ          0x05
#define BITS_DLPF_CFG_5HZ           0x06
#define BITS_DLPF_CFG_2100HZ_NOLPF  0x07
#define BITS_DLPF_CFG_MASK          0x07
#define BIT_INT_ANYRD_2CLEAR        0x10
#define BIT_RAW_RDY_EN              0x01
#define BIT_I2C_IF_DIS              0x10
#define BIT_INT_STATUS_DATA         0x01



//-----------------------------------------------------------------------------
//
MPU6000::MPU6000(int cs_pin)
{
  _spi_cs_pin = cs_pin;
}


//-----------------------------------------------------------------------------
//
void MPU6000::Init()
{
  WriteRegister(MPUREG_PWR_MGMT_1, BIT_H_RESET);
  delay(100);

  WriteRegister(MPUREG_PWR_MGMT_1, MPU_CLK_SEL_PLLGYROZ);
  delay(1);

  WriteRegister(MPUREG_USER_CTRL, BIT_I2C_IF_DIS);
  delay(1);

  WriteRegister(MPUREG_SMPLRT_DIV, 0x04);               // Sample rate = 200Hz
  delay(1);
  
  WriteRegister(MPUREG_CONFIG, BITS_DLPF_CFG_98HZ);     // DLPF = 98Hz (low pass filter)
  delay(1);

  WriteRegister(MPUREG_GYRO_CONFIG, BITS_FS_2000DPS);   // Gyro scale 2000/s
  delay(1);

  WriteRegister(MPUREG_ACCEL_CONFIG, 0x08);             // Accel scele 4g (4096LSB/g)
  delay(1);

  WriteRegister(MPUREG_INT_ENABLE, BIT_RAW_RDY_EN);         // INT: Raw data ready
  delay(1);

  WriteRegister(MPUREG_INT_PIN_CFG, BIT_INT_ANYRD_2CLEAR);  // INT: Clear on any MPU6000_Read
  delay(1);
}


//-----------------------------------------------------------------------------
//
void MPU6000::Read(int32_t values[])
{
  int32_t accelX;
  int32_t accelY;
  int32_t accelZ;

  int32_t gyroX;
  int32_t gyroY;
  int32_t gyroZ;

  int32_t temp;

  uint8_t lowByte;
  uint8_t highByte;


  digitalWrite(_spi_cs_pin, LOW);

  SPI.transfer(MPUREG_ACCEL_XOUT_H | 0x80);

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  accelX   = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  accelY   = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  accelZ   = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  temp     = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  gyroX    = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  gyroY    = (highByte << 8) | lowByte;

  highByte = SPI.transfer(0);
  lowByte  = SPI.transfer(0);
  gyroZ   = (highByte << 8) | lowByte;

  digitalWrite(_spi_cs_pin, HIGH);

  cli();
  values[0] = accelX;
  values[1] = accelY;
  values[2] = accelZ;
  values[3] = temp;
  values[4] = gyroX;
  values[5] = gyroY;
  values[6] = gyroZ;
  sei();
}


//-----------------------------------------------------------------------------
//
void MPU6000::WriteRegister(uint8_t reg, uint8_t val)
{
  uint8_t dump;


  digitalWrite(_spi_cs_pin, LOW);
  
  dump = SPI.transfer(reg);
  dump = SPI.transfer(val);
  
  digitalWrite(_spi_cs_pin, HIGH);
}


//-----------------------------------------------------------------------------
//
int MPU6000::ReadRegister(uint8_t reg)
{
  uint8_t dump;
  uint8_t value;


  digitalWrite(_spi_cs_pin, LOW);

  dump  = SPI.transfer(reg|0x80);
  value = SPI.transfer(0);

  digitalWrite(_spi_cs_pin, HIGH);

  return value;
}